<?php
/*
 * This file is part of Web Pro Manager.
 * 
 * Web Pro Manager is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Web Pro Manager is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Web Pro Manager.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author The Web Pro Miami, Inc. <info@thewebpro.com>
 * @copyright The Web Pro Miami, Inc. <info@thewebpro.com>, http://www.thewebpro.com/
 * @license http://www.gnu.org/licenses/gpl.html GNU GPL
 * @package WebProManager
*/

// Icon and image wizard, v1.0.1
// 1.0.1 - added config nTempImgPath,nImg_MakeIcon: updated paths in nImg_AddText,nIconCoverPath to point to this config ref
/*
Required: imagemagick 6.3 and GD Lib

=== Available Image Multi-Functions:
- nImg_MakeIcon($nOutFileName,$nColor="#FF0000",$nText="")
- nImg_MakeRoundThumb($nInFileName,$nOutFileName,$nThumbSize=128,$nRadius=23)
- nImg_MakeThumb($nInFileName,$nOutFileName,$nThumbSize)

=== Available Image Editing Functions:
- nImg_AddDropShadow($nInFileName,$nOutFileName,$nOpacity=60,$nBlur=2,$nX=2,$nY=2)
- nImg_AddCover($nInFileName,$nOutFileName,$nCoverFile)
- nImg_AddRoundBorder($nInFileName,$nOutFileName,$nRadius=15,$nBorderSize=2,$nBorderColor="#000000")
- nImg_Colorize($nInFileName,$nOutFileName,$nColor="#FFFFFF")
- nImg_MakeRoundCorners($nInFileName,$nOutFileName,$nRadius=15)
- nImg_AddText($nInFileName,$nOutFileName,$nText,$nHexColor,$nFontSize=16,$nShadow=0)
- nImg_ConvertGIF2JPG($nInFileName,$bgcolor=false)
- imageComposeAlpha( &$src, &$ovr, $ovr_x, $ovr_y, $ovr_w = false, $ovr_h = false)
- nImg_MakeRoundCornersGD($nInFileName,$nOutFileName,$nRadius="s",$nBGColor="#FFFFFF")

=== Available Image Utility Functions:
- nImg_GetExtension($nImgFile)
- nColorHex2RGB($nColor)
- nGetImgTagHTML($nImgFile)
- nSanitizeImgFilename($nFileName)
*/

// Get path to ImageMagick convert
$nIMPathARRAY = array();
if(isset($HTTP_ENV_VARS['PATH'])){
	$env_vars = explode( ':', $HTTP_ENV_VARS['PATH']);
	$nIMPathARRAY[] = $env_vars[1]."/convert";
}
// Other possible paths
$nIMPathARRAY[] = "convert";
$nIMPathARRAY[] = "/convert";
$nIMPathARRAY[] = "/usr/bin/convert";
$nIMPathARRAY[] = "/usr/local/bin";
$nIMPathARRAY[] = "/usr/local/bin/convert";
foreach($nIMPathARRAY AS $nPath){
	exec("$nPath -version", $out, $rcode);
	if($rcode===0 || $rcode===1){ $nImgMagickPath = $nPath; }
}
if(!$nImgMagickPath){ echo "<strong style=\"color:#CC0000;\"><BR>ImageMagick is not installed. To use Web Pro Manager to it's full potential, please <a href='http://imagemagick.sourceforge.net/http/www/install.html'>install it</a>.</strong><BR><BR>"; }

// CONFIG (paths relative to script calling this one)
$nTempImgPath = "../../usr/system/icons/";
$nIconCoverPath = "../img/icon_cover.png";
if(!file_exists($nTempImgPath)){ $nTempImgPath = "../usr/system/icons/"; }
if(!file_exists($nIconCoverPath)){ $nIconCoverPath = "img/icon_cover.png"; }

// Test config
if(!file_exists($nIconCoverPath)){ print "CONFIG ERROR: File referenced in 'nIconCoverPath' does not exist: $nIconCoverPath<BR>"; exit; }
if(!file_exists($nTempImgPath)){ print "CONFIG ERROR: Folder referenced in 'nTempImgPath' does not exist: $nTempImgPath<BR>"; exit; }

putenv('GDFONTPATH=' . realpath('.'));

function nImg_MakeIcon($nOutFileName,$nColor="#FF0000",$nText=""){
	global $nIconCoverPath,$nImgMagickPath;
	if(!$nOutFileName){ return; }
	$nSize=60;
	if($nImgMagickPath){
		exec("$nImgMagickPath -size {$nSize}x{$nSize} xc:none -fill \"$nColor\" -draw 'rectangle 0,0 {$nSize},{$nSize}' $nOutFileName");
		if(file_exists($nOutFileName)){
			nImg_MakeRoundCorners($nOutFileName,$nOutFileName,10);
			if($nIconCoverPath){ nImg_AddCover($nOutFileName,$nOutFileName,$nIconCoverPath); } else { print "CONFIG ERROR: NO file referenced in 'nIconCoverPath': $nIconCoverPath.  Please set this in config.<BR>"; }
			nImg_AddDropShadow($nOutFileName,$nOutFileName,80,1,1,1);
			if($nText){ nImg_AddText($nOutFileName,$nOutFileName,$nText,"#fff6c4",11,1); }
		}
	} else {
		$nTmpImg = imagecreatetruecolor($nSize, $nSize);
		$nColorARRAY = nColorHex2RGB($nColor);
		$nTmpColor = imagecolorallocate($nTmpImg,$nColorARRAY[0],$nColorARRAY[1],$nColorARRAY[2]);
		imagefilledrectangle($nTmpImg, 0, 0, $nSize, $nSize, $nTmpColor);
		imagepng($nTmpImg,$nOutFileName);
		imagedestroy($nTmpImg);
		if($nText){ nImg_AddText($nOutFileName,$nOutFileName,$nText,"#fff6c4",11,1); }
	}
}

function nImg_MakeMiniIcon($nOutFileName,$nColor="#FF0000",$nText=""){
	global $nIconCoverPath,$nImgMagickPath;
	if(!$nOutFileName){ return; }
	$nSize=50;
	exec("$nImgMagickPath -size {$nSize}x{$nSize} xc:none -fill \"$nColor\" -draw 'rectangle 0,0 {$nSize},{$nSize}' $nOutFileName");
	nImg_MakeRoundCorners($nOutFileName,$nOutFileName,8);
	if($nIconCoverPath){ nImg_AddCover($nOutFileName,$nOutFileName,$nIconCoverPath); } else { print "CONFIG ERROR: NO file referenced in 'nIconCoverPath': $nIconCoverPath.  Please set this in config.<BR>"; }
	nImg_AddDropShadow($nOutFileName,$nOutFileName,80,1,1,1);
	if($nText){ nImg_AddText($nOutFileName,$nOutFileName,$nText,"#fff6c4",9,1); }
}

function nImg_MakeRoundThumb($nInFileName,$nOutFileName,$nThumbSize=128,$nRadius=23){
	global $nIconCoverPath;
	if(!$nInFileName || !file_exists($nInFileName)){ return; }
	nImg_MakeThumb($nInFileName,$nOutFileName,$nThumbSize);
	nImg_MakeRoundCorners($nOutFileName,$nOutFileName,$nRadius);
	if($nIconCoverPath){ nImg_AddCover($nOutFileName,$nOutFileName,$nIconCoverPath); } else { print "CONFIG ERROR: NO file referenced in 'nIconCoverPath': $nIconCoverPath.  Please set this in config.<BR>"; }
}

function nImg_MakeThumb($nInFileName,$nOutFileName,$nThumbSize){
	global $nImgMagickPath;
	if(!$nInFileName || !file_exists($nInFileName)){ return; }
	$nFirstResize = $nThumbSize*2;
	exec ("$nImgMagickPath -size {$nFirstResize}{$nFirstResize} $nInFileName -thumbnail {$nThumbSize}{$nThumbSize}^ -gravity center -extent {$nThumbSize}{$nThumbSize} $nOutFileName");
}

function nImg_AddDropShadow($nInFileName,$nOutFileName,$nOpacity=60,$nBlur=2,$nX=2,$nY=2){
	global $nImgMagickPath;
	exec("$nImgMagickPath $nInFileName \( +clone -background black -shadow {$nOpacity}x{$nBlur}+{$nX}+{$nY} \) +swap -background none -layers merge +repage $nOutFileName");
}

function nImg_AddCover($nInFileName,$nOutFileName,$nCoverFile){
	global $nTempImgPath,$nImgMagickPath;
	if(!$nInFileName || !file_exists($nInFileName)){ return; }
	if(!$nCoverFile || !file_exists($nCoverFile)){ return; }
	$nImgSize = getimagesize($nInFileName);
	$nImgW = $nImgSize[0]; $nImgH = $nImgSize[1]; $nImgType = $nImgSize[2];
	$nTmpCoverImg = $nTempImgPath."tmp_cover.png";
	exec("$nImgMagickPath $nCoverFile -resize {$nImgW}x{$nImgH} $nTmpCoverImg");
	exec("$nImgMagickPath $nInFileName $nTmpCoverImg -compose over -composite $nOutFileName");
	unlink("$nTmpCoverImg");
}

function nImg_AddRoundBorder($nInFileName,$nOutFileName,$nRadius=15,$nBorderSize=2,$nBorderColor="#000000"){
	global $nTempImgPath,$nImgMagickPath;
	if(!$nInFileName || !file_exists($nInFileName)){ return; }
	$nImgRef = ImageCreateFromPNG("$nInFileName");
	$nRadius = ($nRadius*1.15); // add a little (15%) to radius of border to make it look nicer
	$nImgW = imagesx($nImgRef); $nImgH = imagesy($nImgRef);
	$nTmpRndBrdrImg = $nTempImgPath."tmp_rndbrdr_mask.png";
	exec("$nImgMagickPath -size ".($nImgW+$nBorderSize)."x".($nImgH+$nBorderSize)." xc:none -fill \"$nBorderColor\" -draw \"roundRectangle 0,0 ".($nImgW-1+$nBorderSize).",".($nImgH-1+$nBorderSize)." $nRadius,$nRadius\" $nTmpRndBrdrImg");
	exec("$nImgMagickPath $nTmpRndBrdrImg $nInFileName -geometry +".($nBorderSize/2)."+".($nBorderSize/2)." -compose over -composite $nOutFileName");
	unlink("$nTmpRndBrdrImg");
	ImageDestroy($nImgRef);
}

function nImg_Colorize($nInFileName,$nOutFileName,$nColor="#FFFFFF"){
	if(!$nInFileName || !file_exists($nInFileName)){ return; }
	$nImgRef = ImageCreateFromJPEG("$nInFileName");
	$nColorARRAY = nColorHex2RGB($nColor);
	imagefilter($nImgRef,IMG_FILTER_COLORIZE,$nColorARRAY[0],$nColorARRAY[1],$nColorARRAY[2]);
	imagejpeg($nImgRef,$nOutFileName,85);
	ImageDestroy($nImgRef);
}

function nImg_MakeRoundCorners($nInFileName,$nOutFileName,$nRadius=15){
	global $nImgMagickPath;
	if(!$nInFileName || !file_exists($nInFileName)){ return; }
	exec("$nImgMagickPath $nInFileName \( +clone -threshold -1 -draw 'fill black polygon 0,0 0,$nRadius $nRadius,0 fill white circle $nRadius,$nRadius $nRadius,0' \( +clone -flip \) -compose Multiply -composite \( +clone -flop \) -compose Multiply -composite \) +matte -compose CopyOpacity -composite $nOutFileName");
}

function nImg_AddText($nInFileName,$nOutFileName,$nText,$nHexColor,$nFontSize=16,$nShadow=0){
	global $nTempImgPath,$nImgMagickPath;
	if(!$nInFileName || !file_exists($nInFileName)){ return; }
	$nImgSize = getimagesize($nInFileName);
	$nImgW = $nImgSize[0]; $nImgH = $nImgSize[1]; $nImgType = $nImgSize[2];
	$nTextW = $nImgW*.9; $nTextH = $nImgH*.9;
	$nTmpFontDrk = $nTempImgPath."tmp_font_drk.png";
	$nTmpFont = $nTempImgPath."tmp_font.png";
	if($nImgMagickPath){
		exec ("$nImgMagickPath -size {$nTextW}x{$nTextH} -font Helvetica-Bold -weight Bold -pointsize $nFontSize -background none -fill '$nHexColor' -gravity center caption:\"$nText\" $nTmpFont");
		if($nShadow){
			exec ("$nImgMagickPath -size {$nTextW}x{$nTextH} -font Helvetica-Bold -pointsize $nFontSize -background white -fill black -gravity center caption:\"$nText\" $nTmpFontDrk");
			exec ("$nImgMagickPath $nTmpFontDrk -virtual-pixel edge -blur 0x1 $nTmpFontDrk");
			exec ("$nImgMagickPath $nTmpFontDrk -transparent white $nTmpFontDrk");
			exec ("$nImgMagickPath $nInFileName $nTmpFontDrk -gravity center -geometry -1-3 -compose multiply -composite $nOutFileName");
			exec ("$nImgMagickPath $nOutFileName $nTmpFont -gravity center -geometry -1-3 -compose over -composite $nOutFileName");
			if($nTmpFontDrk && file_exists($nTmpFontDrk)){ unlink("$nTmpFontDrk"); }
		} else {
			exec ("$nImgMagickPath $nInFileName $nTmpFont -gravity center -geometry -1-3 -compose over -composite $nOutFileName");
		}
	} else {
		$nFontSizeGD = $nFontSize-3;
		$nTmpImg = imagecreatefrompng($nInFileName);
		$nColorARRAY = nColorHex2RGB($nHexColor);
		$nTmpColor = imagecolorallocate($nTmpImg,$nColorARRAY[0],$nColorARRAY[1],$nColorARRAY[2]);
		$nFontPath = $_SERVER['DOCUMENT_ROOT']."/manage/inc/arial.ttf";
		$nText = str_replace(" ","\n",$nText);
		imagettftext($nTmpImg, $nFontSizeGD, 0, 10, 20, $nTmpColor, $nFontPath, $nText);
		imagepng($nTmpImg,$nOutFileName);
		imagedestroy($nTmpImg);
	}
	if($nTmpFont && file_exists($nTmpFont)){ unlink("$nTmpFont"); }
}

function nImg_ConvertGIF2JPG($nInFileName,$bgcolor=false){
	if(!$nInFileName){ return; }
	$nOutFileName = str_replace(".gif",".jpg",$nInFileName);
	list($wd, $ht, $tp, $at)=getimagesize($nInFileName);
	$img_src=imagecreatefromgif($nInFileName);
	$img_dst=imagecreatetruecolor($wd,$ht);
	$clr['red']=255;
	$clr['green']=255;
	$clr['blue']=255;
	if(is_array($bgcolor)) $clr=$bgcolor;
	$kek=imagecolorallocate($img_dst,$clr['red'],$clr['green'],$clr['blue']);
	imagefill($img_dst,0,0,$kek);
	imagecopyresampled($img_dst, $img_src, 0, 0, 0, 0, $wd, $ht, $wd, $ht);
	if($nOutFileName){ if($hnd=fopen($nOutFileName,'w')){ fclose($hnd); } }
	imagejpeg($img_dst,$nOutFileName,85);
	imagedestroy($img_dst);
	imagedestroy($img_src);
	return $nOutFileName;
}

function imageComposeAlpha( &$src, &$ovr, $ovr_x, $ovr_y, $ovr_w = false, $ovr_h = false){
	imagecopy($src, $ovr, $ovr_x, $ovr_y, 0, 0, imagesx($ovr), imagesy($ovr));
}

// receives JPG or GIF image and outputs JPG image w/smooth WHITE rounded corners
// (requires GD library)
function nImg_MakeRoundCornersGD($nInFileName,$nOutFileName,$nRadius="s",$nBGColor="#FFFFFF"){
	if(!$nInFileName || !file_exists($nInFileName)){ return; }
	if(nImg_GetExtension($nInFileName) == "gif"){ // if GIF, convert to JPG
		$nInFileName = nImg_ConvertGIF2JPG($nInFileName);
	}
	$nImgRef = ImageCreateFromJPEG("$nInFileName");
	switch($nRadius){
		case "s": $nCornerARRAY = array("5tl.png","5tr.png","5bl.png","5br.png"); break;
		case "m": $nCornerARRAY = array("10tl.png","10tr.png","10bl.png","10br.png"); break;
		case "l": $nCornerARRAY = array("w15tl.png","w15tr.png","w15bl.png","w15br.png"); break;
	}
	$nColorARRAY = nColorHex2RGB($nBGColor);
	foreach($nCornerARRAY AS $nCornerNum=>$nCornerImg){
		$nCornerImgRef = ImageCreateFromPNG($nCornerImg);
		imagealphablending($nCornerImgRef,FALSE);
		imagefilter($nCornerImgRef,IMG_FILTER_COLORIZE,$nColorARRAY[0],$nColorARRAY[1],$nColorARRAY[2]);
		imagesavealpha($nCornerImgRef,TRUE);
		switch($nCornerNum){
			case 0: imageComposeAlpha($nImgRef,$nCornerImgRef,0,0); break;
			case 1: imageComposeAlpha($nImgRef,$nCornerImgRef,(imagesx($nImgRef)-imagesx($nCornerImgRef)),0); break;
			case 2: imageComposeAlpha($nImgRef,$nCornerImgRef,0,(imagesy($nImgRef)-imagesy($nCornerImgRef))); break;
			case 3: imageComposeAlpha($nImgRef,$nCornerImgRef,(imagesx($nImgRef)-imagesx($nCornerImgRef)),(imagesy($nImgRef)-imagesy($nCornerImgRef))); break;
		}
		ImageDestroy($nCornerImgRef);
	}
	imagejpeg($nImgRef,$nOutFileName,95);
	ImageDestroy($nImgRef);
}

function nImg_GetExtension($nImgFile){
	if(!$nImgFile){ return; }
	$nImgInfo = pathinfo($nImgFile);
	$nExt = strtolower($nImgInfo[extension]);
	return $nExt;
}

function nColorHex2RGB($nColor){
	$nColor = str_replace("#","",$nColor);
	if (strlen($nColor) == 6){
		list($r, $g, $b) = array($nColor[0].$nColor[1],$nColor[2].$nColor[3],$nColor[4].$nColor[5]);
	} elseif (strlen($nColor) == 3) {
		list($r, $g, $b) = array($nColor[0].$nColor[0], $nColor[1].$nColor[1], $nColor[2].$nColor[2]);
	} else { return false; }
	$r = hexdec($r); $g = hexdec($g); $b = hexdec($b);
	return array($r, $g, $b);
}

function nGetImgTagHTML($nImgFile){
	if(!$nImgFile){ return; }
	$nImgTagARRAY = getimagesize($nImgFile);
	$nImgTagWH = $nImgTagARRAY[3];
	$nTag = "<img src=\"$nImgFile\" alt=\"\" $nImgTagWH border=\"0\">";
	return $nTag;
}

function nSanitizeImgFilename($nFilename){
	if(!$nFilename){ return; }
	$nBadCharsARRAY = array("amp;",",","!","@","#","$","%","^","&","*","(",")",":",";","/","\"","'","?","<",">","[","]","{","}","|","+","=","~","`");
	$nFilename = str_replace($nBadCharsARRAY,"",$nFilename);
	$nFilename = str_replace(" ","-",$nFilename);
	$nFilename = str_replace("--","-",$nFilename);
	$nFilename = strtolower($nFilename);
	return $nFilename;
}
?>